/*
 * Copyright (c) 2002-2003 Che, Dong Email: chedongATbigfoot.com/chedongATchedong.com
 * $Id: HelloUnicode.java,v 1.3 2003/03/09 08:41:46 chedong Exp $
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

/**
 * Ŀģ
 *     Բַͬ뷽ʽԶֽڱ루ģӰ
 * 룺
 *     Դַ
 * 
 *     1 ղͬ뷽ʽַͬ뷽ʽдļ
 *     2 ղͬ뷽ʽļнַ
 * @author Che, Dong
 */
class HelloUnicode {
    /**
     * main entrance
     * @param args command line arguments
     */
    public static void main(String[] args) {
        String hello = "Hello world ";

        //read from command line input
        if (args.length > 0) {
            hello = args[0];
        }

        try {
            /*
             * 1: Ӳַϵͳȱʡ뷽ʽ룬дļ
             */
            System.out.println(">>>>testing1: write hello world to files<<<<");
            System.out.println("[test 1-1]: with system default encoding="
                + System.getProperty("file.encoding") + "\nstring=" + hello
                + "\tlength=" + hello.length());
            printCharArray(hello);
            writeFile("hello.orig.html", hello);

            //ַGB2312
            hello = new String(hello.getBytes(), "GB2312");
            System.out.println(
                "[test 1-2]: getBytes with platform default encoding and decoding as gb2312:\nstring="
                + hello + "\tlength=" + hello.length());
            writeFile("hello.gb2312.html", hello);
            printCharArray(hello);

            //ַUTF8ֽӡӦֽ
            hello = new String(hello.getBytes("UTF8"));
            System.out.println("[test 1-3]: convert string to UTF8\nstring="
                + hello + "\tlength=" + hello.length());
            writeFile("hello.utf8.html", hello);
            printCharArray(hello);

            /*
             * 2: 1ļжȡղͬʽ
             */
            System.out.println(
                ">>>>testing2: reading and decoding from files<<<<");

            //first file: encoding with system default
            hello = readFile("hello.orig.html");
            System.out.println(
                "[test 2-1]: read hello.orig.html: decoding with system default encoding\nstring="
                + hello + "\tlength=" + hello.length());
            printCharArray(hello);

            //second file: decoding from GBK
            hello = readFile("hello.gb2312.html");
            hello = new String(hello.getBytes(), "GB2312");
            System.out.println(
                "[test 2-2]: read hello.gb2312.html: decoding as GB2312\nstring="
                + hello + "\tlength=" + hello.length());
            printCharArray(hello);

            //third file: decoding from UTF8
            hello = readFile("hello.utf8.html");
            hello = new String(hello.getBytes(), "UTF8");
            System.out.println(
                "[test 2-3]: read hello.utf8.html: decoding as UTF8\nstring="
                + hello + "\tlength=" + hello.length());
            printCharArray(hello);
        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }

    /**
     * print char array
     * @param inStr input string
     */
    public static void printCharArray(String inStr) {
        char[] myBuffer = inStr.toCharArray();

        //list each Charactor in byte value, short value, and UnicodeBlock Mapping
        for (int i = 0; i < inStr.length(); i++) {
            byte b = (byte) myBuffer[i];
            short s = (short) myBuffer[i];
            String hexB = Integer.toHexString(b).toUpperCase();
            String hexS = Integer.toHexString(s).toUpperCase();
            StringBuffer sb = new StringBuffer();

            //print char
            sb.append("char[");
            sb.append(i);
            sb.append("]='");
            sb.append(myBuffer[i]);
            sb.append("'\t");

            //byte value
            sb.append("byte=");
            sb.append(b);
            sb.append(" \\u");
            sb.append(hexB);
            sb.append('\t');

            //short value
            sb.append("short=");
            sb.append(s);
            sb.append(" \\u");
            sb.append(hexS);
            sb.append('\t');

            //Unicode Block
            sb.append(Character.UnicodeBlock.of(myBuffer[i]));

            System.out.println(sb.toString());
        }

        System.out.println();
    }

    /**
     * write content to output file
     * @param fileName output file name
     * @param content  file content to write
     */
    private static void writeFile(String fileName, String content) {
        try {
            File tmpFile = new File(fileName);

            if (tmpFile.exists()) {
                tmpFile.delete();
            }

            FileWriter fw = new FileWriter(fileName, true);
            fw.write(content);
            fw.close();
        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }

    /**
     * read content from input file
     * @param fileName input file name
     * @return String file content
     */
    private static String readFile(String fileName) {
        try {
            BufferedReader fr = new BufferedReader(new FileReader(fileName));
            StringBuffer out = new StringBuffer();
            String thisLine = new String();

            while (thisLine != null) {
                thisLine = fr.readLine();

                if (thisLine != null) {
                    out.append(thisLine);
                }
            }

            fr.close();

            return out.toString();
        } catch (Exception e) {
            System.out.print(e.toString());
            return null;
        }
    }
}
